from pathlib import Path

from .experiment_panel import ExperimentPanel
from .mxbi_panel import MXBIPanel


def config_wizard(
    mxbi_config_path: Path,
    session_config_path: Path,
    options_path: Path,
):
    import sys

    from PySide6.QtWidgets import QApplication

    app = QApplication(sys.argv)

    mxbi_panel = MXBIPanel(mxbi_config_path, options_path)
    experiment_panel = ExperimentPanel(session_config_path, options_path)
    mxbi_panel.accepted.connect(experiment_panel.show)
    experiment_panel.accepted.connect(app.quit)

    mxbi_panel.show()

    app.exec()



from dataclasses import dataclass
from queue import Empty, SimpleQueue

import pygame
from pygame import Event, event
from pymxbi.detector import MockDetector
from pymxbi.detector.detector import DetectionResult, Detector, DetectorEvent

EVT_DETECTOR = pygame.USEREVENT + 1


@dataclass(frozen=True)
class DetectorMsg:
    kind: DetectorEvent
    animal: str | None


class DetectorBridge:
    def __init__(self, detector: Detector) -> None:
        self._detector = detector
        self._q = SimpleQueue()
        self._started = False

    def start(self) -> None:
        if self._started:
            return
        self._started = True

        self._detector.begin()
        self._detector.register_event(DetectorEvent.ANIMAL_ENTERED, self._emit_entered)
        self._detector.register_event(DetectorEvent.ANIMAL_LEFT, self._emit_left)
        self._detector.register_event(DetectorEvent.FAULT_DETECTED, self._emit_fault)

    def _emit(self, kind: DetectorEvent, animal: str | None) -> None:
        self._q.put(DetectorMsg(kind=kind, animal=animal))

    def _emit_entered(self, detection_result: DetectionResult) -> None:
        self._emit(
            DetectorEvent.ANIMAL_ENTERED,
            detection_result.animal_id or detection_result.animal_name,
        )

    def _emit_left(self, detection_result: DetectionResult) -> None:
        self._emit(
            DetectorEvent.ANIMAL_LEFT,
            detection_result.animal_id or detection_result.animal_name,
        )

    def _emit_fault(self, detection_result: DetectionResult) -> None:
        self._emit(
            DetectorEvent.FAULT_DETECTED,
            detection_result.animal_id or detection_result.animal_name,
        )

    def emit_pygame_event(self) -> None:
        while True:
            try:
                msg = self._q.get_nowait()
            except Empty:
                break

            event.post(event.Event(EVT_DETECTOR, msg=msg))

    def manaul_emit(self, animal_idx: int | None = None) -> None:
        if isinstance(self._detector, MockDetector):
            if animal_idx is None:
                self._detector.animal_left()
            else:
                self._detector.animal_present(animal_idx)

    def handle_event(self, event: Event) -> None:
        if event.type == pygame.KEYDOWN:
            match event.key:
                case pygame.K_0:
                    self.manaul_emit(0)
                case pygame.K_1:
                    self.manaul_emit(1)
                case pygame.K_2:
                    self.manaul_emit(2)
                case pygame.K_3:
                    self.manaul_emit(3)
                case pygame.K_4:
                    self.manaul_emit(4)
                case pygame.K_5:
                    self.manaul_emit(5)
                case pygame.K_l:
                    self.manaul_emit()



import sys

from loguru import logger

from ..path import LOG_PATH

logger.remove()

logger.add(sys.stderr, level="DEBUG")

logger.add(
    f"{LOG_PATH}/mxbi.log",
    rotation="10 MB",
    retention="7 days",
    compression="zip",
    encoding="utf-8",
    level="DEBUG",
    serialize=True,
)



import pymxbi
from loguru import logger
from pymxbi import MXBI, MXBIModel

from .config_store import ConfigStore
from .models.session import SessionConfig


def build_mxbi(mxbi_config_path, session_config_path) -> MXBI:

    mxbi_config = ConfigStore(mxbi_config_path, MXBIModel).value
    session_config = ConfigStore(session_config_path, SessionConfig).value

    mxbi = pymxbi.build_mxbi(mxbi_config, logger)
    mxbi.register_animal(
        {animal.rfid_id: animal.name for animal in session_config.animals}
    )

    return mxbi



from pathlib import Path

from pygame import Event, Surface

from ..config_store import ConfigStore
from ..models.session import Options
from .scene_protocol import SceneProtocol


class SceneManager:
    _scenes: dict[str, type[SceneProtocol]] = {}

    def __init__(self) -> None:
        self.current: SceneProtocol | None = None
        self._pending: SceneProtocol | None = None

    def persist(self, path: Path) -> None:
        options_store = ConfigStore(path, Options)

        options_store.value.stages = list(self._scenes.keys())
        options_store.save()

    def register(
        self, scene: dict[str, type[SceneProtocol]] | list[type[SceneProtocol]]
    ) -> None:
        if isinstance(scene, dict):
            self._scenes.update(scene)
        elif isinstance(scene, list):
            for s in scene:
                self._scenes[s.__name__.lower()] = s
        else:
            raise ValueError("Invalid scene type")

    @property
    def scenes(self) -> dict[str, type[SceneProtocol]]:
        return self._scenes

    def switch(self, scene: type[SceneProtocol], defer: bool = True) -> None:
        if defer:
            self._pending = scene()
        else:
            self._switch(scene())

    def _switch(self, next_scene: SceneProtocol) -> None:
        prev = self.current
        if prev is not None and prev.running:
            prev.quit()

        self.current = next_scene
        next_scene.start()

    def apply_pending(self) -> None:
        if self._pending is None:
            return
        next_scene = self._pending
        self._pending = None
        self._switch(next_scene)

    def handle_event(self, event: Event) -> None:
        if self.current:
            self.current.handle_event(event)

    def update(self, dt_s: float) -> None:
        if self.current:
            self.current.update(dt_s)

    def draw(self, screen: Surface) -> None:
        if self.current:
            self.current.draw(screen)



from pymxbi.detector import FusionContinuousDetectorModel
from PySide6.QtWidgets import QLabel, QLineEdit

from ..device_card import DeviceCard


class BeambreakDetectorCard(DeviceCard[FusionContinuousDetectorModel]):
    def __init__(self):
        super().__init__()

        self.set_title("Beambreak Detector")

        label_pin = QLabel("Pin")
        self._line_pin = QLineEdit()
        self._line_pin.setPlaceholderText("Pin")
        self.layout_config.addRow(label_pin, self._line_pin)

    def load_config(self, model: FusionContinuousDetectorModel) -> None:
        self.checkbox_enabled.setChecked(model.enabled)
        self.line_device_id.setText(str(model.id))
        self._line_pin.setText(str(model.pin))

    @property
    def result(self) -> FusionContinuousDetectorModel:
        return FusionContinuousDetectorModel(
            enabled=self.checkbox_enabled.isChecked(),
            id=int(self.line_device_id.text()),
            pin=int(self._line_pin.text()),
        )



from pathlib import Path

from ..config_store import ConfigStore
from ..models.animal import Animal, StageState
from ..models.session import DailySessionIdStore, Session, SessionConfig


def init_session(session_config_path: Path, session_counter_path: Path) -> Session:
    session_config = ConfigStore(session_config_path, SessionConfig).value
    store = DailySessionIdStore(session_counter_path)

    animal_dict: dict[str, Animal] = {}
    for animal_config in session_config.animals:
        train_state = StageState(
            stage_name=animal_config.stage, level=animal_config.level
        )
        animal_state = Animal(
            rfid_id=animal_config.rfid_id,
            name=animal_config.name,
        )
        animal_state.set_current_stage(train_state)
        animal_dict[animal_config.name] = animal_state

    session = Session(
        session_id=store.session_id,
        experimenter=session_config.experimenter,
        reward_type=session_config.reward_type,
        send_email=False,
        sync_data=False,
        note=session_config.note,
        animals=animal_dict,
    )
    session.start()

    return session



from PySide6.QtCore import Qt, Signal
from PySide6.QtWidgets import QComboBox, QFormLayout, QLabel, QLineEdit, QMenu

from ...models.animal import AnimalConfig
from ...models.session import Options
from .card import CardFrame


class AnimalCard(CardFrame):
    remove_requested = Signal()

    def __init__(self, parent, animals: dict[str, str], options: Options):
        super().__init__(parent=parent, object_name="card")
        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.customContextMenuRequested.connect(self._on_context_menu)

        items = list(animals.items())
        self._animal_ids = [animal_id for animal_id, _name in items]
        self._animal_names = [name for _animal_id, name in items]

        layout = QFormLayout(self)
        layout.setContentsMargins(8, 8, 8, 8)

        label_animal_name = QLabel("animal", self)
        self.combo_animal_name = QComboBox(self)
        self.combo_animal_name.addItems(self._animal_names)
        self.combo_animal_name.currentIndexChanged.connect(self._on_animal_changed)
        layout.addRow(label_animal_name, self.combo_animal_name)

        lable_animal_id = QLabel("id", self)
        self.line_animal_id = QLineEdit(self)
        self.line_animal_id.setReadOnly(True)
        self.line_animal_id.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        layout.addRow(lable_animal_id, self.line_animal_id)

        label_stage = QLabel("stage", self)
        items = options.stages
        self.combo_stage = QComboBox(self)
        self.combo_stage.addItems(items)
        self.combo_stage.setCurrentText("idle")
        layout.addRow(label_stage, self.combo_stage)

        label_level = QLabel("level", self)
        self.combo_level = QComboBox(self)
        self.combo_level.addItems(["0", "1"])
        self.combo_level.setCurrentText("0")
        layout.addRow(label_level, self.combo_level)

        self._sync_animal_id()

    def _on_animal_changed(self, _index: int) -> None:
        self._sync_animal_id()

    def _sync_animal_id(self) -> None:
        index = self.combo_animal_name.currentIndex()
        if index < 0 or index >= len(self._animal_ids):
            self.line_animal_id.setText("")
            return
        self.line_animal_id.setText(self._animal_ids[index])

    def animal_id(self) -> str:
        return self.line_animal_id.text()

    def animal_name(self) -> str:
        return self.combo_animal_name.currentText()

    def _on_context_menu(self, pos):
        menu = QMenu(self)
        action = menu.addAction("Remove")
        action.triggered.connect(lambda _checked=False: self.remove_requested.emit())
        menu.exec(self.mapToGlobal(pos))

    def load_config(self, config: AnimalConfig) -> None:
        self.combo_animal_name.setCurrentText(config.name)
        self.combo_stage.setCurrentText(config.stage)
        self.combo_level.setCurrentText(str(config.level))

    @property
    def result(self) -> AnimalConfig:
        return AnimalConfig(
            rfid_id=self.animal_id(),
            name=self.animal_name(),
            stage=self.combo_stage.currentText(),
            level=int(self.combo_level.currentText()),
        )



from .mxbi import build_mxbi
from .mxbiflow import MXBIFlow, get_mxbiflow
from .ui.utils import config_wizard
from .utils.init_session import init_session

__all__ = [
    "MXBIFlow",
    "get_mxbiflow",
    "config_wizard",
    "build_mxbi",
    "init_session",
]



from pathlib import Path

# TODO: XDG Base Directory Specification
ROOT_DIR_PATH = Path(__file__).parents[2]

CONFIG_DIR_PATH = ROOT_DIR_PATH / "config"
CONFIG_SESSION_FILENAME = "config_session.json"
CONFIG_SESSION_PATH = CONFIG_DIR_PATH / CONFIG_SESSION_FILENAME

OPTIONS_SESSION_FILENAME = "options_session.json"
OPTIONS_SESSION_PATH = CONFIG_DIR_PATH / OPTIONS_SESSION_FILENAME

ANIMAL_DB_FILENAME = "animal_db.json"
ANIMAL_DB_PATH = CONFIG_DIR_PATH / ANIMAL_DB_FILENAME

CROSS_MODAL_CONFIG_FILENAME = "config_cross_modal.json"
CROSS_MODAL_CONFIG_PATH = CONFIG_DIR_PATH / CROSS_MODAL_CONFIG_FILENAME

DATA_DIR_PATH = ROOT_DIR_PATH / "data"

LOG_PATH = ROOT_DIR_PATH / "log"

SAMBA_MOUNT_PATH = ROOT_DIR_PATH / "samba_mount"
SAMBA_BACKUP_DIR_NAME = "backup"
SAMBA_BACKUP_DIR_PATH = SAMBA_MOUNT_PATH / SAMBA_BACKUP_DIR_NAME

SERVICE_DIR_NAME = "services"
SERVICE_DIR_PATH = ROOT_DIR_PATH / SERVICE_DIR_NAME
MOUNT_SERVICE_NAME = "mount.service"
MOUNT_SERVICE_PATH = SERVICE_DIR_PATH / MOUNT_SERVICE_NAME
SYNC_SERVICE_NAME = "sync.service"
SYNC_SERVICE_PATH = SERVICE_DIR_PATH / SYNC_SERVICE_NAME

SESSION_COUNTER_PATH = CONFIG_DIR_PATH / "session_counter.json"

ASSETS_DIR_PATH = ROOT_DIR_PATH / "assets"
ASSET_CLICKER_PATH = ASSETS_DIR_PATH / "clicker.wav"
